function WtForUpdate = UpdateParticleWorld(scan_local)
global Particles

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle
%                    .WinnerIndex
%                    .World{p} -- A world structure defining the map maintained by particle 'p' (World structure described below)

% ---

% Structure World
%                .N                 -- Number of elements in the world
%                .wt(k)         -- The weight of the particle when this element was added (i.e. weight of the element)
%                .element{k}        -- A structure defining a line/curve
%                           .Center     -- A central point of the k-th element
%                           .RMax       -- An upperbound on the 'radius' of the k-th element
% Option 1:
%                           .type       =  'LineSegment'
%                           .ends       -- A 2x2 matrix each each column representing a vertex
%                           .etc
%                               .SegmentVector
%                               .SegmentLength
%                               .SegmentUnitVector
%                               .an
%                               .bn
%                               .cn
% Option 2:
%                           .type       = 'CircularArk'
%                           .c          -- center
%                           .r          -- radius
%                           .anglerange -- a 1x2 vector of [startAngle, endAngle]
%                           .etc
%                               .endPoints -- 2x2 matrix

% =============================================================================

% Break up scan points into clusters
ClusterIndices = ClusterScanPoints(scan_local);

% Construct the elements in local coordinates
elements_local = [];
for a = 1:length(ClusterIndices)
    elements_local = [elements_local, FitElementsToClusterPoints(scan_local(:,ClusterIndices{a}))];
end

% Add the elements to the maps of individual particles
for p = 1:Particles.N
    
    elements_global = TransformElements(elements_local, Particles.pos(1,p), Particles.pos(2,p), Particles.th(p));
    [Particles.World{p}, WtForUpdate(p)] = MergeElementsToWorld(elements_global, Particles.World{p}, Particles.wt(p), Particles.pos(:,p));
    
end
    
    
% % %     % Obtain the estimated global coordinates of the scan points
% % %     ScanPoints_global = TransformScanPoints(scan_local, Particles.pos(1,p), Particles.pos(2,p), Particles.th(p));
% % % 
% % %     % Determine which elements in the particle's world are the relevant ones
% % %     belongTags = sum( ( Particles.World{p}.Center - repmat(Particles.pos(:,p),[1 size(Particles.World{p}.Center,2)]) ).^2) <= (Particles.World{p}.RMax + 4);
% % %     WorldElementIndices = find(belongTags);
% % %     
% % %     % Compute distances from relevant world elements
% % %     [distsMin, elementIndices, distsMat] = ComputeMinDist_PointsWorld(ScanPoints_global, Particles.World{p}, WorldElementIndices);
% % %     
% % %     % Process each cluster to merge new points with the closest world elements or create new world elements
% % %     for c = 1:length(ClusterIndices)
% % %         thisCluster_points = ScanPoints_global(:,ClusterIndices{c});
% % %         thisCluster_distsMin = distsMin(ClusterIndices{c});
% % %         thisCluster_closestElementIndices = elementIndices(ClusterIndices{c});
% % %         
% % %         ClusterPointIndices = ProcessClusterBelonging(thisCluster_distsMin, thisCluster_closestElementIndices);
% % %         
% % %         modalElementIndex = Mode_PositiveInteger(thisCluster_elementIndices);
% % %         thisCluster_DistFromModalElement = distsMat(modalElementIndex, ClusterIndices{c});
% % %         [newElements, centers, RMaxs] = ComputeElements_ElementCluster(Particles.World{p}.element{modalElementIndex}, thisCluster_points);
        