function VGraph_ComputeLeastCostPath
%{
******************************************************************************************
*                                                                                        *
*    A* search with homotopy class constraints                                           *
*    Copyright (C) 2010  Subhrajit Bhattacharya                                          *
*                                                                                        *
*    This program is free software: you can redistribute it and/or modify                *
*    it under the terms of the GNU General Public License as published by                *
*    the Free Software Foundation, either version 3 of the License, or                   *
*    (at your option) any later version.                                                 *
*                                                                                        *
*    This program is distributed in the hope that it will be useful,                     *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of                      *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                       *
*    GNU General Public License for more details <http://www.gnu.org/licenses/>.         *
*                                                                                        *
*                                                                                        *
*    Contact: subhrajit@gmail.com, http://fling.seas.upenn.edu/~subhrabh/                *
*                                                                                        *
*                                                                                        *
******************************************************************************************
%}


global VGraph
% Structure VGraph
%                 .ActiveIndices        -- A vector containing the indices of the active clusters ( = [1,2,ObstaclesInSight+2])
%                 .VertexCluster{i}     -- All the vertices associated with cluster i
%                                           ( i=1 -> start position - the only dynamic cluster
%                                             i=2 -> goal position
%                                             i=p -> Obstacle number p-2 )
%                                  .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%
%                 .InterClusterEdges{i,j}  -- Edges joining clusters i and j [IMPORTANT: i <= j ]
%                                        .isComputed -- Indicates whether these edges have been computed before (Check using isfield)
%                                        .Edges -- A 6xE matrix of edges along with cost
%                                                                   = [ v_i1    v_i2    ...   ;
%                                                                       v_j1    v_j2    ...   ;
%                                                                       cost_1  cost_2  ...   ;
%                                                                       a_1     a_2     ...   ;
%                                                                       b_1     b_2     ...   ;
%                                                                       c_1     c_2     ...   ; ]
%                                                                   v_kp is the index of the vertex in VGraph.VertexCluster(k).Vertices
%
%                 .LeastCostPath(k) -- contains information about the k-th segment of the path
%                                  .i    -- Index of cluster from where the edge originates
%                                  .v_i  -- Vertex index of cluster i
%                                  .j    -- Index of cluster where the edge ends
%                                  .v_j  -- Vertex index of cluster j
%                                  .e    -- Edge index in VGraph.InterClusterEdges{i,j}.Edges
%
% c_k = norm( VGraph.VertexCluster{i}.Vertices(1:2,v_ik) - VGraph.VertexCluster{j}.Vertices(1:2,v_jk) )
%

VGraph.ActiveIndices = sort(VGraph.ActiveIndices);

% % Teating encode/decode
% for i = 1:length(VGraph.VertexCluster)
%     for v = 1:size(VGraph.VertexCluster{i}.Vertices,2)
%         encoded_v = EncodeVertexIndex(i, v);
%         [ClusterNo, VertexNo] = DecodeVertexIndex(encoded_v);
%         encoded_v,
%         disp([i v; ClusterNo VertexNo]);
%     end
% end
% 

t0 = clock;
[PathNodes, LVal] = search;
disp(sprintf('Search time: %f seconds', etime(clock,t0)));

for k=1:size(PathNodes,1)-1
    VGraph.LeastCostPath(k).i = PathNodes(k,1);
    VGraph.LeastCostPath(k).v_i = PathNodes(k,2);
    VGraph.LeastCostPath(k).j = PathNodes(k+1,1);
    VGraph.LeastCostPath(k).v_j = PathNodes(k+1,2);
    if PathNodes(k,1)<=PathNodes(k+1,1)
        ii=PathNodes(k,1); jj=PathNodes(k+1,1);
        v_ii=PathNodes(k,2); v_jj=PathNodes(k+1,2);
    else
        ii=PathNodes(k+1,1); jj=PathNodes(k,1);
        v_ii=PathNodes(k+1,2); v_jj=PathNodes(k,2);
    end
    EdgesCopy = VGraph.InterClusterEdges{ii,jj}.Edges;
    VGraph.LeastCostPath(k).e = find( EdgesCopy(1,:)==v_ii & EdgesCopy(2,:)==v_jj );
end

disp(sprintf('LVal of path = %f+%fi', real(LVal), imag(LVal)));
%pause;

%{
Edges = [];
for a = 1:length(VGraph.ActiveIndices)
    for b = a:length(VGraph.ActiveIndices)
        
        i = VGraph.ActiveIndices(a);
        j = VGraph.ActiveIndices(b);
        
        if ~isempty(VGraph.InterClusterEdges{i,j}.Edges)
            vertices_i = EncodeVertexIndex(i, VGraph.InterClusterEdges{i,j}.Edges(1,:));
            vertices_j = EncodeVertexIndex(j, VGraph.InterClusterEdges{i,j}.Edges(2,:));
            costs_ij = VGraph.InterClusterEdges{i,j}.Edges(3,:);

            Edges = [Edges; [vertices_i', vertices_j', costs_ij']];
        end
    end
end

Edges = [Edges ; Edges(:,[2 1 3])];

s = EncodeVertexIndex(1,1);
t = EncodeVertexIndex(2,1);
% Edges(:,[1 2]),
% s,
% t,
%[dSP, sp] = grShortPath(Edges, s, t);
sp = ShortestPath_Dijkstra(Edges, s, t);
%sp,
% if length(sp) < 2
%     VGraph.LeastCostPath = [];
%     return;
% end


VGraph.LeastCostPath = [];
[ClusterNo_start, VertexNo_start] = DecodeVertexIndex(sp(1));
for a = 2:length(sp)
    [ClusterNo_end, VertexNo_end] = DecodeVertexIndex(sp(a));
    i = min([ClusterNo_start, ClusterNo_end]);
    j = max([ClusterNo_start, ClusterNo_end]);
    if i==ClusterNo_start && j==ClusterNo_end
        VGraph.LeastCostPath(a-1).direction = 1;
        v_i = VertexNo_start;
        v_j = VertexNo_end;
    else
        VGraph.LeastCostPath(a-1).direction = -1;
        v_i = VertexNo_end;
        v_j = VertexNo_start;
    end
    VGraph.LeastCostPath(a-1).i = i;
    VGraph.LeastCostPath(a-1).v_i = v_i;
    VGraph.LeastCostPath(a-1).j = j;
    VGraph.LeastCostPath(a-1).v_j = v_j;
    %VGraph.InterClusterEdges{ClusterNo_start,ClusterNo_end},
    if i==j
        VGraph.LeastCostPath(a-1).e = find( (VGraph.InterClusterEdges{i,j}.Edges(1,:)==v_i ...
                                           & VGraph.InterClusterEdges{i,j}.Edges(2,:)==v_j) | ...
                                            (VGraph.InterClusterEdges{i,j}.Edges(1,:)==v_j ...
                                           & VGraph.InterClusterEdges{i,j}.Edges(2,:)==v_i));
    else
        VGraph.LeastCostPath(a-1).e = find( VGraph.InterClusterEdges{i,j}.Edges(1,:)==v_i ...
                                            & VGraph.InterClusterEdges{i,j}.Edges(2,:)==v_j );
    end
    ClusterNo_start = ClusterNo_end;
    VertexNo_start = VertexNo_end;
end


% In the following functions ClusterNo should be a scalar
% VertexNo can be a vector
% ----------------------------------------------

function VertexIndex = EncodeVertexIndex(ClusterNo, VertexNo)
global VGraph

VertexIndex = zeros(size(VertexNo));
OnesOfVertexNoSize = ones(size(VertexNo));

for a = VGraph.ActiveIndices
    if a == ClusterNo
        VertexIndex = VertexIndex + VertexNo;
        break;
    else
        VertexIndex = VertexIndex + size(VGraph.VertexCluster{a}.Vertices, 2)*OnesOfVertexNoSize;
    end
end

% ----------------------------------------------

function [ClusterNo, VertexNo] = DecodeVertexIndex(VertexIndex)
% Elements of VertexIndex MUST belong to the same cluster
global VGraph

OnesOfVertexIndexSize = ones(size(VertexIndex));
VertexNo = VertexIndex;

for a = VGraph.ActiveIndices
    if all(VertexNo <= size(VGraph.VertexCluster{a}.Vertices, 2))
        ClusterNo = a;
        break;
    else
        VertexNo = VertexNo - size(VGraph.VertexCluster{a}.Vertices, 2)*OnesOfVertexIndexSize;
    end
end
%}
