function wt = ComputeParticleWeights(ScanPoints_local, PredictedParticles)

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle

% ScanPoints_local is a 2xM matrix with each column representing a scan point
% in the local coordinate frame of the particle
% The transformation from local to global coordinates is governed by the function
% ScanPoints_global = TransformScanPoints(ScanPoints_local, x, y, th)

ScanPoints_global_Dump = [];
PresentDumpSize = 0;
ScanPointDumpMarkers = [];
for a = 1:PredictedParticles.N
    ScanPoints_global_this = TransformScanPoints(ScanPoints_local, PredictedParticles.pos(1,a), ...
                                                    PredictedParticles.pos(2,a), PredictedParticles.th(a));
    NewPointsCount = size(ScanPoints_global_this, 2);
    ScanPointDumpMarkers(1,a) = PresentDumpSize + 1;
    PresentDumpSize = PresentDumpSize + NewPointsCount;
    ScanPointDumpMarkers(2,a) = PresentDumpSize;
    %ScanPoints_global_Dump = [ ScanPoints_global_Dump, ScanPoints_global_this ];
    ScanPoints_global_Dump(:,ScanPointDumpMarkers(1,a):ScanPointDumpMarkers(2,a)) = ScanPoints_global_this;
    
end

dists_Dump = ComputeMinDistFromWalls(ScanPoints_global_Dump);

for a = 1:PredictedParticles.N
    ThisParticleDists = dists_Dump(:,ScanPointDumpMarkers(1,a):ScanPointDumpMarkers(2,a));
    ThisParticleMeanSqDists = mean(ThisParticleDists .^ 2);
    wt(a) = 1/ThisParticleMeanSqDists;
end


% ====================================================

function ScanPoints_global = TransformScanPoints(ScanPoints_local, x, y, th)

laserOffset = 0.063; % whatever!
ScanPoints_local(1,:) = ScanPoints_local(1,:) + laserOffset;
transMat = [ cos(th), -sin(th), x ; sin(th), cos(th), y ; 0, 0, 1 ];
ScanPoints_global = transMat * [ ScanPoints_local ; ones([1 size(ScanPoints_local,2)]) ];
ScanPoints_global = ScanPoints_global(1:2, :);
