function InitParticles = InitiateParticles(N, ParticleBoundary, PossibleLocation)
global UObstacles Boundary

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle

% ParticleBoundary is either a 2x2 matrix - [ xmin xmax ; ymin ymax ];
%                         or a 3x2 matrix - [ xmin xmax ; ymin ymax ; thmin thmax ];
% It denotes a range within which to generate the particles
% If omitted or empty, ParticleBoundary = [Boundary ; -pi pi ];

% Should check if the particles lie inside the UObsatacles (not implemented presently)

if nargin < 3
    PossibleLocation = [];
end

if nargin < 2
    ParticleBoundary = [Boundary ; -pi pi ];
elseif size(ParticleBoundary,1) == 2
    ParticleBoundary(3,:) = [-pi, pi];
end


x_sigma = (ParticleBoundary(1,2)-ParticleBoundary(1,1)) / 6;
y_sigma = (ParticleBoundary(2,2)-ParticleBoundary(2,1)) / 6;
th_sigma = (ParticleBoundary(3,2)-ParticleBoundary(3,1)) / 6;
if ~isempty(PossibleLocation)
    x_mean = PossibleLocation(1);
    y_mean = PossibleLocation(2);
    th_mean = PossibleLocation(3);
else
    x_mean = (ParticleBoundary(1,2)+ParticleBoundary(1,1)) / 2;
    y_mean = (ParticleBoundary(2,2)+ParticleBoundary(2,1)) / 2;
    th_mean = (ParticleBoundary(3,2)+ParticleBoundary(3,1)) / 2;
end

Nexcess = round(1.5*N);

InitParticles.pos = [];
InitParticles.th = [];
InitParticles.N = 0;
while InitParticles.N < N
    % If PossibleLocation is not empty, the following should be generated out of a Gaussian distribution
    if isempty(PossibleLocation)
        xs = x_mean + (rand(1,Nexcess)-0.5)*x_sigma*6;
        ys = y_mean + (rand(1,Nexcess)-0.5)*y_sigma*6;
        ths = th_mean + (rand(1,Nexcess)-0.5)*th_sigma*6;
    else
        xs = x_mean + randn(1,Nexcess)*x_sigma;
        ys = y_mean + randn(1,Nexcess)*y_sigma;
        ths = th_mean + randn(1,Nexcess)*th_sigma;
    end

    % Check which points lie inside the Obstacles or outside ParticleBoundary and eliminate them
    pointValid = not(isPointInsideObstacle([xs;ys]));
    validIndices = find(pointValid);
    validIndices = validIndices(1:min([N,length(validIndices)]));
    chosenCount = length(validIndices);
    
    % Choose only N or less ValidPoints
    InitParticles.pos = [InitParticles.pos, [xs(validIndices);ys(validIndices)]];
    InitParticles.th = [InitParticles.th, ths(validIndices)];
    InitParticles.N = InitParticles.N + chosenCount;

end


% ----------------------------------------------------
