function SMAN_example(spec_filename)

global Control
global VGraph Obstacles
global NowCoord GoalCoord

% Structure Control
%                  .params
%                         .RobotRadius
%                         .FeedbackLinearizationDistance
%                         .Clearance_KnownObstacles
%                         .Clearance_GeneratedObstacles
%                         .PathDiscretizationResolution
%                  .PathPoints
%                             .NowIndex
%                             .points
Control.params.RobotRadius = 0.1;
Control.params.FeedbackLinearizationDistance = 0.05;
Control.params.Clearance_KnownObstacles = 0.05; % 0.2;
Control.params.Clearance_GeneratedObstacles = 0.15;

Control.params.PathDiscretizationResolution = 0.01;

Control.params.SearchAheadDistForClosestPoint = 1.0;
Control.params.TargetPointDistanceAhead = 0.03; %0.2;

Control.params.RobotSpeed = 0.2;


%Robot_Communicate('initiate');

InitiateUObstacle(spec_filename);
DrawScene('initiate');
DrawScene('GoalPoint');
DrawScene('UObstacles_Geometry');

% -----------------------------------------------

GenerateObstacles;
VGraph_Initiate;
DrawScene('Obstacles');
DrawScene('NowPoint');

VGraph_ComputeLeastCostPath;
ValidPath = VGraph_CheckLeastCostPath;

while ~ValidPath.isValid
    for a = 1:length(ValidPath.Segment)
        if ~ValidPath.Segment(a).isValid
            VGraph_RemoveBlockedEdges(VGraph.LeastCostPath(a));
            for b=1:length(ValidPath.Segment(a).BlockObstacles)
                VGraph_AddObstacle(ValidPath.Segment(a).BlockObstacles(b));
            end
        end
    end
    VGraph_ComputeLeastCostPath;
    ValidPath = VGraph_CheckLeastCostPath;
end

DiscretizeLeastCostPath;
DrawScene('Obstacles');
DrawScene('VGraph');
DrawScene('Path');
DrawScene('NowPoint');
pause;
% -----------------------------------------------

firstTime = 1;
t0 = clock;
lastVel = [0, 0];
while (1)

    %{
    
    if etime(clock, t0) > 1 | firstTime
        Robot_Communicate('setVelocity', [0, 0]);
        pause(0.1);

        isNewPath = 0;

        % The following needs to be done only at intervals (but definitely the first time loop starts)

        % Read data from robot - position and laser scan
        robotData = Robot_Communicate('getData');
        RobotSetVelocity(Control.params.RobotSpeed / 2);
        %Robot_Communicate('setVelocity', lastVel/2);

        % Update obstacles from laser scans
        [UpdatedUObstacleIndices, CheckedUObstaclesIndices] = UpdateDynamicUObstacles(robotData.LaserPoints);
        if ~isempty(UpdatedUObstacleIndices) | firstTime

            GenerateObstacles(UpdatedUObstacleIndices);
            ValidPath = VGraph_CheckLeastCostPath(UpdatedUObstacleIndices);

            %%%^VGraph_ResetCluster(UpdatedUObstacleIndices+2); % ###

            %VGraph_ComputeLeastCostPath; %%

            if ~ValidPath.isValid
                VGraph_ResetCluster(1:length(VGraph.InterClusterEdges));
                %for a = 1:length(ValidPath.Segment)
                %VGraph_RemoveBlockedEdges(VGraph.LeastCostPath(a)); % Previous path no more remains valid!
                %end
                %for a = UpdatedUObstacleIndices
                %VGraph_AddObstacle(a); % Previous path no more remains valid!
                %end
                VGraph_ResetCluster(1); % ###
                NowCoord = robotData.p;
                VGraph_Initiate;
                VGraph_ComputeLeastCostPath;
                ValidPath = VGraph_CheckLeastCostPath;
                isNewPath = 1;
                %DrawScene('Path'); pause;
            end

            while ~ValidPath.isValid
                for a = 1:length(ValidPath.Segment)
                    if ~ValidPath.Segment(a).isValid
                        VGraph_RemoveBlockedEdges(VGraph.LeastCostPath(a));
                        for b=1:length(ValidPath.Segment(a).BlockObstacles)
                            VGraph_AddObstacle(ValidPath.Segment(a).BlockObstacles(b));
                        end
                    end
                end
%                 DrawScene('UObstacles_Geometry',UpdatedUObstacleIndices);
%                 DrawScene('UObstacles_Points',CheckedUObstaclesIndices);
%                 DrawScene('Obstacles',UpdatedUObstacleIndices);
%                 DrawScene('VGraph');
%                 DrawScene('NowPoint');
                VGraph_ComputeLeastCostPath;
                ValidPath = VGraph_CheckLeastCostPath;
                %DrawScene('Path'); pause;
            end

            if isNewPath | firstTime
                DiscretizeLeastCostPath;
                DrawScene('Path');
            end

            DrawScene('UObstacles_Geometry',UpdatedUObstacleIndices);
            DrawScene('UObstacles_Points',CheckedUObstaclesIndices);
            DrawScene('Obstacles',UpdatedUObstacleIndices);
            DrawScene('VGraph');
            DrawScene('NowPoint');

        end
        
        t0 = clock;
    end

    %}

    RobotSetVelocity;

    firstTime = 0;
    
    if size(Control.PathPoints.points,2) - Control.PathPoints.NowIndex < 4
        break;
    end
    
end

Robot_Communicate('setVelocity', [0, 0]);

% ---------------------------

function RobotSetVelocity(speed)
global Control

    if nargin < 1
        speed = Control.params.RobotSpeed;
    end

    robotData = Robot_Communicate('getData_Pose');
    currentPos = robotData.p + Control.params.FeedbackLinearizationDistance*[cos(robotData.theta); sin(robotData.theta)];
    searchAheadPointIndices = ...
        Control.PathPoints.NowIndex:min([ Control.PathPoints.NowIndex + round(Control.params.SearchAheadDistForClosestPoint/Control.params.PathDiscretizationResolution), ...
                                          size(Control.PathPoints.points,2) ]);
    distSqFromAheadPoints = sum( (Control.PathPoints.points(:,searchAheadPointIndices) - repmat(currentPos, [1 length(searchAheadPointIndices)])) .^ 2);
    [nearestPtDistSq, nearestPtIndex] = min(distSqFromAheadPoints);

    Control.PathPoints.NowIndex = searchAheadPointIndices(nearestPtIndex);
    
    targetPointIndex = min([ Control.PathPoints.NowIndex+round(Control.params.TargetPointDistanceAhead/Control.params.PathDiscretizationResolution), ...
                             size(Control.PathPoints.points,2) ]);
    targetPoint = Control.PathPoints.points(:,targetPointIndex);
    
    globalVel = targetPoint - currentPos;
    globalVel = speed * globalVel / norm(globalVel);
    
    localVel = [cos(robotData.theta), sin(robotData.theta); -sin(robotData.theta), cos(robotData.theta)] * globalVel;
    lastVel = [localVel(1), localVel(2)/Control.params.FeedbackLinearizationDistance];
    Robot_Communicate('setVelocity', lastVel);
    