function data = Robot_Communicate(option, params)
persistent init_struct robot0_id truth0_id laser0_id x
persistent start_time


switch option
    
    case 'initiate'

        init_struct = struct('id', {{{'sim','default'}}}, 'interface', {{'sim'}});
        % Add our interfaces to the init_struct
        %[init_struct, robot0_id] = createPosition(init_struct, 'robot0');
        [init_struct, robot0_id] = createPosition(init_struct, 'scarab0');
        if any(strcmp('truth',params))
            [init_struct, truth0_id] = createTruth(init_struct, 'truth_robot0');
        end
        if any(strcmp('laser',params))
            [init_struct, laser0_id] = createLaser(init_struct, 'laser1');
        end
        % Initialize the interface. Gazebo must be running before initializing.
        x = gazeboAPIClient('init', init_struct);
        % Need to enable motors before using
        gazeboAPIPosition('cmdEnableMotors',robot0_id, 1.0);
        % Get current simulation time
        start_time = gazeboAPISim('delay', 0, 0);
        data = [];

    case 'getData'
        
        laser_localPos = [0.0622340; 0.0];
        data_truth = gazeboAPITruth('truth', truth0_id);
        data.p = [data_truth.pos(1); data_truth.pos(2)];
        data.theta = data_truth.rot_euler(3);
        data_laser = gazeboAPILaser('laser', laser0_id);
        angles = data_laser.min_angle + (data_laser.max_angle-data_laser.min_angle)*(0:(data_laser.range_count-1))/(data_laser.range_count-1);
        ValidIndices = data_laser.ranges < (data_laser.max_range * 0.6);
        angles = angles(ValidIndices);
        ranges = data_laser.ranges(ValidIndices);
        data.LaserPoints = [];
        data.LaserPoints(1,:) = data_truth.pos(1) + laser_localPos(1)*cos(data_truth.rot_euler(3)) - laser_localPos(2)*sin(data_truth.rot_euler(3)) + ranges.*cos(angles+data_truth.rot_euler(3));
        data.LaserPoints(2,:) = data_truth.pos(2) + laser_localPos(1)*sin(data_truth.rot_euler(3)) + laser_localPos(2)*cos(data_truth.rot_euler(3)) + ranges.*sin(angles+data_truth.rot_euler(3));
        
    case 'getData_Pose'

        %laser_localPos = [0.0622340; 0.0];
        data_truth = gazeboAPITruth('truth', truth0_id);
        data.p = [data_truth.pos(1); data_truth.pos(2)];
        data.theta = data_truth.rot_euler(3);
        
    case 'getData_LaserLocal'

        laser_localPos = [0.0622340; 0.0];
        data_laser = gazeboAPILaser('laser', laser0_id);
        angles = data_laser.min_angle + (data_laser.max_angle-data_laser.min_angle)*(0:(data_laser.range_count-1))/(data_laser.range_count-1);
        ValidIndices = data_laser.ranges < (data_laser.max_range * 0.6);
        angles = angles(ValidIndices);
        ranges = data_laser.ranges(ValidIndices);
        data.LaserPoints_local = [];
        data.LaserPoints_local(1,:) = ranges.*cos(angles) + laser_localPos(1);
        data.LaserPoints_local(2,:) = ranges.*sin(angles) + laser_localPos(2);

    case 'setVelocity'

        % params(1) -- forward velocity ; params(2) -- angular velocity
        setVelocity(robot0_id, params(1), params(2));
        
end
