function LUL_example(spec_filename)

global Control
global VGraph Obstacles Particles
global NowCoord GoalCoord

SetSearchPaths;

disp('Do you want to compare the results with truth data? (Truth interface must be present in the world file!)')
CompareWithTruth = input('Type 1(yes) or 0(no) : ');
disp(sprintf('\nPlease wait while initial path planning and localization is performed.\nRobot will start moving in a while!\n'));

% Structure Control
%                  .params
%                         .RobotRadius
%                         .FeedbackLinearizationDistance
%                         .Clearance_KnownObstacles
%                         .Clearance_GeneratedObstacles
%                         .PathDiscretizationResolution
%                  .PathPoints
%                             .NowIndex
%                             .points
Control.params.RobotRadius = 0.1;
Control.params.FeedbackLinearizationDistance = 0.05;
Control.params.Clearance_KnownObstacles = 0.15; %0.05; % 0.2;
Control.params.Clearance_GeneratedObstacles = 0.15;

Control.params.PathDiscretizationResolution = 0.01;

Control.params.SearchAheadDistForClosestPoint = 1.0;
Control.params.TargetPointDistanceAhead = 0.2; %0.03; %0.2;

Control.params.RobotSpeed = 0.05; % 0.2
Control.params.MaxTurnRate = 0.8;


if CompareWithTruth
    Robot_Communicate('initiate', {'truth','laser'});
    CompareData.EstimatedPos = [];
    CompareData.TruePos = [];
else
    Robot_Communicate('initiate', 'laser');
end

InitiateUObstacle(spec_filename);
DrawScene('initiate');
DrawScene('GoalPoint');
DrawScene('UObstacles_Geometry');

% -----------------------------------------------

GenerateObstacles;
VGraph_Initiate;
DrawScene('Obstacles');
DrawScene('NowPoint');

% VGraph_ComputeLeastCostPath;
% ValidPath = VGraph_CheckLeastCostPath;
% 
% while ~ValidPath.isValid
%     for a = 1:length(ValidPath.Segment)
%         if ~ValidPath.Segment(a).isValid
%             VGraph_RemoveBlockedEdges(VGraph.LeastCostPath(a));
%             for b=1:length(ValidPath.Segment(a).BlockObstacles)
%                 VGraph_AddObstacle(ValidPath.Segment(a).BlockObstacles(b));
%             end
%         end
%     end
%     VGraph_ComputeLeastCostPath;
%     ValidPath = VGraph_CheckLeastCostPath;
% end
% 
% DiscretizeLeastCostPath;
% DrawScene('Obstacles');
% DrawScene('VGraph');
% DrawScene('Path');
% DrawScene('NowPoint');

% -----------------------------------------------

GenerateWorld;
% Perform Initial Localization

rotDirections = [1, -1, 1e-6];
rotTimes = [0.5, 1.0, 1.0];
SurviveRate = [5, 5, 5];
FinalCount = [500, 500, 500]; %[1000, 1000, 1000];
for a = 1:length(rotTimes)
    
    disp(sprintf('Running pass %d of %d for initial Localization...', a, length(rotTimes)));
    %RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(0,0,1e-6,1);
    RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(0,0,1e-6,1);
    pause(0.2);
    ParticlesAfterDownsample.SurviveRate = SurviveRate(a);
    ParticlesAfterDownsample.FinalCount = FinalCount(a);
    RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(1,1,Control.params.MaxTurnRate*rotDirections(a), ...
                                                                                                1,ParticlesAfterDownsample);
    %Particles.N,
    estPos = DrawScene('Particles');
    pause(rotTimes(a));
    
end

% -----------------------------------------------

% Recompute path!

RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(0,0,1e-6,1);

%clear VGraph;
VGraph_Initiate;

VGraph_ComputeLeastCostPath;
ValidPath = VGraph_CheckLeastCostPath;

while ~ValidPath.isValid
    for a = 1:length(ValidPath.Segment)
        if ~ValidPath.Segment(a).isValid
            VGraph_RemoveBlockedEdges(VGraph.LeastCostPath(a));
            for b=1:length(ValidPath.Segment(a).BlockObstacles)
                VGraph_AddObstacle(ValidPath.Segment(a).BlockObstacles(b));
            end
        end
    end
    VGraph_ComputeLeastCostPath;
    ValidPath = VGraph_CheckLeastCostPath;
end

DiscretizeLeastCostPath;
DrawScene('Obstacles');
DrawScene('VGraph');
DrawScene('Path');
DrawScene('NowPoint');


%pause;

% -----------------------------------------------

% Main loop
% NowCoord = [];
% firstTime = 1;
% t0 = clock;
% lastVel = [0, 0];

disp(sprintf('\nRobot will run soon!\n'));

while (1)
    
    %RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(0,0,1e-6);
    pause(0.2);
    RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(1,1);
    motionStart_timestamp = clock;
    while etime(clock, motionStart_timestamp) <= 0.5
        %pause(0.5);
        RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(0,0);
    end
    
    estPos = DrawScene('Particles');
    %Particles.N,
    %pause;
    if CompareWithTruth
        CompareData.EstimatedPos(:,end+1) = estPos;
        dataPose = Robot_Communicate('getData_Pose');
        CompareData.TruePos(:,end+1) = dataPose.p;
    end

    %firstTime = 0;
    
    if size(Control.PathPoints.points,2) - Control.PathPoints.NowIndex < 4
        break;
    end
    
end


Robot_Communicate('setVelocity', [0, 0]);

if CompareWithTruth
    figure;
    hold on;
    axis equal;
    title('Comparison between true trajectory and trajectory estimated by particle filter');
    plot(CompareData.TruePos(1,:), CompareData.TruePos(2,:), 'b-');
    plot(CompareData.EstimatedPos(1,:), CompareData.EstimatedPos(2,:), 'r-');
    legend('True Trajectory', 'Estimated Trajectory');
end

clear all;

% ---------------------------
