function Particles_Propagate(vel, dt, noise)
global Particles

% The StartParticles are propagated with velocity 'vel' for time 'dt' to obtain new particle positions
% vel(1) is forward velocity ; vel(2) is omega

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle
%                    .WorldBelonging  -- World.N x Particles.N matrix

if nargin < 3
    noise = [0; 0];
end

vel_1 = vel(1) + randn([1 Particles.N])*noise(1)*vel(1);
vel_2 = vel(2) + randn([1 Particles.N])*noise(2)*vel(2);

newTh = Particles.th + vel_2*dt;

shiftVector = repmat(vel_1./vel_2, [2 1]) .* [ sin(newTh)-sin(Particles.th) ; ...
                                                -cos(newTh)+cos(Particles.th) ];

Particles.pos = Particles.pos + shiftVector;
Particles.th = newTh;
