function ValidPath = VGraph_CheckLeastCostPath(chkObstacles)
global VGraph Obstacles

% Structure Obstacles
%                    .N                -- number of obstacles
%                    .obs{i}           -- cell array of structure that defines i-th obstacle
%                            .shape    -- Can be 'circle' or 'polygon' (present code supports only polygons)
%                       polygon:
%                            .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%

% Structure VGraph
%                 .ActiveIndices        -- A vector containing the indices of the active clusters ( = [1,2,ObstaclesInSight+2])
%                 .VertexCluster{i}     -- All the vertices associated with cluster i
%                                           ( i=1 -> start position - the only dynamic cluster
%                                             i=2 -> goal position
%                                             i=p -> Obstacle number p-2 )
%                                  .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%
%                 .InterClusterEdges{i,j}  -- Edges joining clusters i and j [IMPORTANT: i <= j ]
%                                        .isComputed -- Indicates whether these edges have been computed before (Check using isfield)
%                                        .Edges -- A 6xE matrix of edges along with cost
%                                                                   = [ v_i1    v_i2    ...   ;
%                                                                       v_j1    v_j2    ...   ;
%                                                                       cost_1  cost_2  ...   ;
%                                                                       a_1     a_2     ...   ;
%                                                                       b_1     b_2     ...   ;
%                                                                       c_1     c_2     ...   ; ]
%                                                                   v_kp is the index of the vertex in VGraph.VertexCluster(k).Vertices
%
%                 .LeastCostPath(k) -- contains information about the k-th segment of the path
%                                  .i    -- Index of cluster from where the edge originates
%                                  .v_i  -- Vertex index of cluster i
%                                  .j    -- Index of cluster where the edge ends
%                                  .v_j  -- Vertex index of cluster j
%                                  .e    -- Edge index in VGraph.InterClusterEdges{i,j}.Edges
%
% c_k = norm( VGraph.VertexCluster{i}.Vertices(1:2,v_ik) - VGraph.VertexCluster{j}.Vertices(1:2,v_jk) )
%

% Structure ValidPath
%                    .isValid    -- 1 if the whole path is unblocked, 0 otherwise
%                    .Segment(k) -- Structure describing the k-the segment of the path
%                               .isValid -- 1 if the segment is unvlocked, 0 otherwise
%                               .BlockObstacles -- a row vector with indices of obstacles that block the segment

ValidPath.isValid = 1;
if nargin < 1
    chkObstacles = setdiff(1:Obstacles.N, VGraph.ActiveIndices-2);
    %chkObstacles = 1:Obstacles.N;
end

% if length(VGraph.LeastCostPath) < 1
%     ValidPath.isValid = 0;
%     return;
% end

for a = 1:length(VGraph.LeastCostPath)
    
%     if VGraph.LeastCostPath(a).i == VGraph.LeastCostPath(a).j
%         ValidPath.Segment(a).isValid = 1;
%         ValidPath.Segment(a).BlockObstacles = [];
%         continue;
%     end
    
    ThisEdge = VGraph.LeastCostPath(a);
    
    ValidPath.Segment(a).isValid = 1;
    ValidPath.Segment(a).BlockObstacles = [];
    
    for obsNo = chkObstacles
        clusterNo = obsNo + 2;
        if any(clusterNo==[ThisEdge.i, ThisEdge.j])
            result = 1;
        else
        
%         if VGraph.InterClusterEdges{clusterNo,clusterNo}.isComputed
%             result = isIntersect_EdgeCluster(ThisEdge, obsNo);
%         else
        %ThisEdge,
            result = isIntersect_EdgeObstacle(ThisEdge, obsNo); % This will also create VGraph.InterClusterEdges{clusterNo,clusterNo} if not present
%         end
        end
        
        if result
            ValidPath.Segment(a).BlockObstacles = [ValidPath.Segment(a).BlockObstacles, obsNo];
            ValidPath.Segment(a).isValid = 0;
        end
    end
    
end

ValidPath.isValid = all([ValidPath.Segment(:).isValid]);
