/*
 *  Gazebo - Outdoor Multi-Robot Simulator
 *  Copyright (C) 2003  
 *     Nate Koenig & Andrew Howard
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * Desc: EXAMPLE ONLY for dynamically-loaded models
 * Author: Andrew Howard
 * Date: 14 Jul 2004
 * CVS: $Id: Mer1.hh,v 1.2 2004/09/09 18:31:41 inspectorg Exp $
 */

#ifndef SCARAB_HH
#define SCARAB_HH

#include <gazebo.h>
#include <gazebo/Body.hh>
#include <gazebo/Model.hh>

// Forward declarations
class Body;
class HingeJoint;
class BallJoint;
class RayProximity;

class Scarab : public Model
{
  // Constructor
  public: Scarab( World *world );
  
  // Destructor
  public: virtual ~Scarab();

  // Load stuff
  public: int Load( WorldFile *file, WorldFileNode *node );

  // Load ODE stuff
  private: int OdeLoad( WorldFile *file, WorldFileNode *node );
  
  // Initialize stuff
  public: int Init( WorldFile *file, WorldFileNode *node );

  // Finalize stuff
  public: int Fini();
  
  // Do updates
  public: virtual void Update( double step );

  // Update the odometry
  private: void UpdateOdometry( double step );

  // Get commands from the external interface
  private: void GetPositionCmd();
  
  // Update the data in the external interface
  private: void PutPositionData();

  // Update the data in the power interface
  private: void PutPowerData();

  // The canonical body
  private: Body *tma;

  // Update parameters
  private: double updatePeriod, updateTime;

  // Robot parameters
  private: double wheelSep, wheelDiam;

	// Noise paramaters
private: double cov_v, cov_w, cov_vw;
private: double input_cov_v, input_cov_w, input_cov_vw;
private: gsl_rng *r;  // gsl random number generator
  

  // ODE components
  private: Body *chassis;
  private: Body *wheels[2];
  private: HingeJoint *wheelJoints[4];
  private: Body *casterWheels[2];
  private: BallJoint *casterJoints[2];

  // External interface
  private: gz_position_t *position_iface;
  private: gz_power_t *power_iface;

  // Wheel speeds (left and right wheels are paired)
  private: int enableMotors;
  private: double wheelSpeed[2];

  // Odometric pose estimate
  private: double odomPose[3];

  // Odometric speed estimate
  private: double odomVel[3];

  // Battery level
  private: double batteryLevel;

  // Coefficients of battery discharge curve
  private: double batteryCurve[2];

  // Color variables
  private: GzColor body_color, wheel_color;
};

#endif
